unit Plan96P;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Light, Material, Transfrm, cOpenGL, OpenGL, MMsystem, ExtCtrls;

type
  TForm1 = class(TForm)
    OpenGL1: TOpenGL;
    TRansformLight: TTRansform;
    TRansformFloor: TTRansform;
    TRansformWall: TTRansform;
    MaterialLight: TMaterial;
    MaterialFloor: TMaterial;
    MaterialWall: TMaterial;
    Light1: TLight;
    TRansformPole: TTRansform;
    MaterialPole: TMaterial;
    MaterialOpen: TMaterial;
    TRansformGlobe: TTRansform;
    MaterialGlobe: TMaterial;
    TRansformLand: TTRansform;
    MaterialLand: TMaterial;
    TRansformDust: TTRansform;
    MaterialDust: TMaterial;
    Timer1: TTimer;
    TRansformCanvas: TTRansform;
    MaterialCanvas: TMaterial;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure TRansformLightPaint(Sender: TObject);
    procedure TRansformFloorPaint(Sender: TObject);
    procedure TRansformWallPaint(Sender: TObject);
    procedure TRansformPolePaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TRansformGlobePaint(Sender: TObject);
    procedure TRansformLandPaint(Sender: TObject);
    procedure TransformDustPaint(Sender: TObject);
    procedure TransformCanvasPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
        theta,
     ang, angl, rspeed, speed, tran, rot, alt, wx, wy, wz: GLfloat;
     frames, elapsed, start, pass, small, last_small, f1, f2: integer;
     MY_UP, MY_DOWN, MY_LEFT, MY_RIGHT, MY_W, MY_S: Boolean;
     procedure InitDust;
     procedure DrawDust;
     procedure VectorDust;
     procedure SphereDust;
     procedure SpreadDust;
     procedure CollectDust;
     procedure BuildTerrain;
     procedure DrawTerrain;
     procedure BuildWall;
     procedure Cube;
     procedure CylinderBase;
     procedure Cylinder;
     procedure Opening;
     procedure Idle(Sender: TObject; var Done: boolean);
     procedure Adjust;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
   Level = 0;
   Border = 0;
   ColorComps =3;
   MAX_DUST = 1440;
   RADI = 1.0;

type
   TRGB = packed record
      r,g,b: byte;
   end;

   TWrap = array[0..0] of TRGB;

   TPoint = packed record
     x,y,z: GLfloat;
   end;

   TTerrainRow=array[1..66] of TPoint;
   TTerrain = array[-32..32] of TTerrainRow;
   TDust = packed array[1..MAX_DUST,0..1] of TPoint;
var
   pi180: double = pi/180;
   circleX, circleY : array [0..35] of glFloat;
   wall: array[1..28] of TPoint;
   sBulk, tBulk,
   sBrick, tBrick,
   sWorld, tWorld, sGrass, tGrass: GLsizei;
   globe, world, bulk, brick, grass: pointer;
   ter: TTerrain;
   Dust: TDust;
//   dlTerrain: GLuint;

function ReadBitmap(const FilePath: string; var sWidth,tHeight:GLsizei): pointer;
const
   szh = SizeOf(TBitmapFileHeader);
   szi = SizeOf(TBitmapInfoHeader);
var
   bmpfile: file;
   bfh: TBitmapFileHeader;
   bmi: TBitmapInfoHeader;
   t: byte;
   x1, size: integer;
begin
   assignFile(bmpfile, FilePath);
   reset(bmpfile,1);
   size := FileSize(bmpfile)-szh-szi;
   blockread(bmpfile, bfh, szh);
   if Bfh.BfType<>$4D42 then
      raise EInvalidGraphic.Create('invalid bitmap');
   blockread(bmpfile,bmi,szi);
   with bmi do
   begin
      sWidth := biWidth;
      tHeight := biHeight;
   end;
   getmem(result,size);
   blockRead(bmpfile,result^,size);
   for x1 := 0 to sWidth * tHeight -1 do
   with TWrap(result^)[x1] do
   begin
      t := r;
      r := b;
      b := t;
   end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   glPixelStorei(GL_UNPACK_ALIGNMENT,1);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
   glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);//GL_DECAL);
   bulk := readBitmap('Fig13-07.bmp',sBulk,tBulk);
   Brick := readBitmap('Fig13-10.bmp',sBrick,tBrick);
  // World := readBitmap('AreaMap.bmp',sWorld,tWorld);

//   World := readBitmap('Fig13-21.bmp',sWorld,tWorld);
   World := readBitmap('earth2.bmp',sWorld,tWorld);

  // World := readBitmap('Fig13-16.bmp',sWorld,tWorld);
   Grass := readBitmap('Fig13-19.bmp',sGrass,tGrass);
{
   glEnable(GL_LINE_SMOOTH);
   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 }
   Application.OnIdle := Idle;
   cylinderBase;
   buildWall;
   buildTerrain;
   InitDust;
   SphereDust;
   globe := gluNewQuadric;
   gluQuadricDrawStyle(globe,GLU_FILL);
   gluQuadricTexture(globe,byteBool(GL_TRUE));
   start := TimeGetTime;
   speed := 0.2;
   rspeed := 7.0;

   MY_UP := False;
   MY_DOWN := False;
   MY_LEFT := False;
   MY_RIGHT := False;
   MY_W := False;
   MY_S := False;

   pass := 0;
   last_small := -1;
   small := 0;
   f1 := 1;
   f2 := 2;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   case Key of
      VK_UP: MY_UP := TRUE;
      VK_DOWN: MY_DOWN := TRUE;
      VK_LEFT: MY_LEFT := TRUE;
      VK_RIGHT: MY_RIGHT := TRUE;
      VK_ESCAPE: Close;
      ord('R'): Begin
      if ssShift in Shift then
         rspeed := rspeed +0.01
      else begin
         if rspeed > 0.01 then
            rspeed := rspeed -0.01;
         end;
      end;
      ord('X'): begin
         if ssShift in Shift then
            speed := speed +0.001
         else begin
            if speed > 0.001 then
               speed := speed -0.001;
            end;
         end;
      ord('W'): begin
             MY_W := True;
             end;
      ord('S'): begin
            MY_S := True;
            end;
      end; {case}
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   case Key of
      VK_UP: begin
             MY_UP := False;
             tran := 0.0;
             end;
      VK_DOWN: begin
             MY_DOWN := False;
             tran := 0.0;
             end;
      VK_LEFT: begin
             MY_LEFT := False;
             rot := 0.0;
             end;
      VK_RIGHT: begin
             MY_RIGHT := False;
             rot := 0.0;
             end;
      ord('W'): begin
             MY_W := False;
             alt := 0.0;
             end;
      ord('S'): begin
             MY_S := False;
             alt := 0.0;
             end;
   end;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
   glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;

   MaterialOpen.Apply;
   Opening;

   glRotatef(angl,0.0,1.0,0.0);
   glTranslatef(wx,wy,wz);

   TransformLight.Apply;
   TransformWall.Apply;
   TransformPole.Apply;
   TransformGlobe.Apply;
//   TransformFloor.Apply;
   TransFormCanvas.Apply;
   TransformDust.Apply;
   TransformLand.Apply;

   OpenGL1.Swap;
   OpenGL1.ErrorCheck('formpaint');
end;

procedure TForm1.FormResize(Sender: TObject);
begin
   if OpenGL1.Smaller then
      FormPaint(Sender);

end;

procedure TForm1.TRansformLightPaint(Sender: TObject);
begin
   Light1.Apply;
   MaterialLight.Apply;
   Cube;

end;

procedure TForm1.TRansformFloorPaint(Sender: TObject);
begin
   MaterialFloor.Apply;
   glNormal3f(0.0,1.0,0.0);
   glBegin(GL_POLYGON);
      glVertex3f(-29.0,0.0,-29.0);
      glVertex3f(-29.0,0.0,+29.0);
      glVertex3f(+29.0,0.0,+29.0);
      glVertex3f(+29.0,0.0,-29.0);
   glEnd;

end;

procedure TForm1.TRansformWallPaint(Sender: TObject);
var
   ix: integer;
   s: GLfloat;
   t: array[false..true] of GLfloat;
   tt: boolean;
begin
   glEnable(GL_TEXTURE_2D);
   glTexImage2D(GL_TEXTURE_2D,Level,ColorComps,sBrick,tBrick,Border,
                GL_RGB,GL_UNSIGNED_BYTE,Brick);

   MaterialWall.Apply;

   s := 0.0;
   t[false] := 0.0;
   t[true] := 4.0;
   tt := false;

   glBegin(GL_TRIANGLE_STRIP);
     for ix := 1 to 14 do
     with wall[ix] do
     begin
        glNormal3f(0.0,0.0,1.0);
        glTexCoord2f(s,t[tt]);
        glVertex3f(x,y,z);
        if tt then
           s := s+2.0;
        tt := not tt;
     end;
     for ix := 15 to 28 do
     with wall[ix] do
     begin
        glNormal3f(0.0,0.0,-1.0);
        glTexCoord2f(s,t[tt]);
        glVertex3f(x,y,z);
        if tt then
           s := s+2.0;
        tt := not tt;
     end;

     for ix := 1 to 2 do
     with wall[ix] do
     begin
        glNormal3f(0.0,-1.0,0.0);
        glTexCoord2f(s,t[tt]);
        glVertex3f(x,y,z);
        if tt then
           s := s+2.0;
        tt := not tt;
     end;
  glEnd;
  glDisable(GL_TEXTURE_2D);

//   Cube;
end;

procedure TForm1.InitDust;
const
   frac = 10000;
var
   count: integer;
begin
   Randomize;
   for count := 1 to MAX_DUST do
   begin
      with Dust[count,0] do
      begin
         x := (Random(12000)-6000)/frac;
         y := (Random(12000)-6000)/frac;
         z := (Random(12000)-6000)/frac;
      end;
      with Dust[count,1] do
      begin
         x := 0;//(Random(12000)-6000)/frac /1000;
         y := 0;//(Random(12000)-6000)/frac /1000;
         z := 0;//(Random(12000)-6000)/frac /1000;
      end;
   end;
end;

procedure TForm1.DrawDust;
var
   count: integer;
begin
   glPointSize(2.0);
   glBegin(GL_POINTS);
   for count := 1 to MAX_DUST do
   begin
      with Dust[count,0] do
      begin
         glNormal3f(x,y,z);
         glVertex3f(x,y,z);
      end;
   end;
   glEnd;

   glBegin(GL_LINES);
      with Dust[f1,0] do
      begin
         glNormal3f(-x,-y,-z);
         glVertex3f(0.0,0.0,0.0);
         glVertex3f(x,y,z);
      end;
   glEnd;
   glBegin(GL_LINES);
      with Dust[f2,0] do
      begin
         glNormal3f(-x,-y,-z);
         glVertex3f(0.0,0.0,0.0);
         glVertex3f(x,y,z);
      end;
   glEnd;
end;

function Normalize(N: TPoint):TPoint;
var
   d, m: GLFloat;
begin
   d := sqrt(N.x*N.x + N.y*N.y + N.z*N.z);
   m := RADI / d;
   Result.x := N.x * m;
   Result.y := N.y * m;
   Result.z := N.z * m;
end;

function Distance(N1, N2: TPoint): GLFloat;
var
   xd,yd,zd: GLFloat;
begin
   xd := N2.x-N1.x;
   yd := N2.y-N1.y;
   zd := N2.z-N1.z;
   Result := sqrt(xd*xd + yd*yd + zd*zd);
end;

procedure TForm1.SphereDust;
var
   count : integer;
begin
   for count := 1 to MAX_DUST do
         Dust[count,0] := Normalize(Dust[count,0]);
end;

procedure TForm1.SpreadDust;
var
   count, count2, s1, s2: integer;
   x1,y1,z1, x2,y2,z2,
   xd,yd,zd, d, m,
   percent: GLFloat;
begin
   m := 2.0;
   s1 := 0;
   s2 := 0;

   for count := 1 to MAX_DUST do
   begin
      for count2 := 1 to MAX_DUST do
      begin
         if count <> count2 then
         begin
            d := Distance(Dust[count,0], Dust[count2,0]);

            if d < m then
            begin
               m := d;
               s1 := count;
               s2 := count2;
            end;

         end;
      end;
   end;

   if (m < 1.0) and
      (s1 <> 0) and
      (s2 <> 0) and
      (s1 <> s2) then
   begin
      inc(pass);
      f1 := s1;
      f2 := s2;
            x1 := Dust[s1,0].x;
            y1 := Dust[s1,0].y;
            z1 := Dust[s1,0].z;

            x2 := Dust[s2,0].x;
            y2 := Dust[s2,0].y;
            z2 := Dust[s2,0].z;

      // move point 2 away by 20%
      percent := 1.2;
      if m > 0.9 then
         percent :=  0.1110 / m *10;

      xd := x2 - x1;
      yd := y2 - y1;
      zd := z2 - z1;

      dust[s2,0].x := (xd * percent) + x1;
      dust[s2,0].y := (yd * percent) + y1;
      dust[s2,0].z := (zd * percent) + z1;

      // move point 1 away by 20%
      xd := x1 - x2;
      yd := y1 - y2;
      zd := z1 - z2;

      dust[s1,0].x := (xd * percent) + x2;
      dust[s1,0].y := (yd * percent) + y2;
      dust[s1,0].z := (zd * percent) + z2;

      // normalize distances for 2 points
      Dust[s1,0] := Normalize(Dust[s1,0]);
      Dust[s2,0] := Normalize(Dust[s2,0]);
   end;

   // info calc for caption text output
   d := Distance(Dust[s1,0], Dust[s2,0]);

   form1.Caption := 'small: '+IntToStr(small)
                   +' pass: '+IntToStr(pass)
                   +'   ' +IntToStr(s1)
                   +',  ' +IntToStr(s2)
                   +'   ' +FloatToStr(d);
end;

procedure TForm1.CollectDust;
const
   OPTIM = 0.1220;
var
   count, count2,
   s1,s2,s3,s4: Integer;
   d, m,m3,m4, dx,dy,dz, reduce: GLFloat;
begin
   last_small := Small;
   s1 := 0;
   s2 := 0;
   s3 := 0;
   s4 := 0;

   for count := 1 to MAX_DUST do
   begin
      m := 1.2;

      for count2 := 1 to MAX_DUST do
      begin
         d := Distance(Dust[count,0], Dust[count2,0]);

         if count <> count2 then
            if d < m then
            begin
               s4 := s3;
               m4 := m3;
               s3 := s2;
               m3 := m;

               m := d;
               s1 := count;
               s2 := count2;
            end;
      end;

      if m > OPTIM then
      begin
         inc(Small);
         dx := Dust[s1,0].x - Dust[s2,0].x;
         dy := Dust[s1,0].y - Dust[s2,0].y;
         dz := Dust[s1,0].z - Dust[s2,0].z;
         reduce := OPTIM / m;

         Dust[s2,0].x := (dx * reduce) +Dust[s2,0].x;
         Dust[s2,0].y := (dy * reduce) +Dust[s2,0].y;
         Dust[s2,0].z := (dz * reduce) +Dust[s2,0].z;

         Dust[s2,0] := Normalize(Dust[s2,0]);
      end;

      if (m3 > OPTIM) and
         (s3 <> 0)  then
      begin
         inc(Small);
         dx := Dust[s1,0].x - Dust[s3,0].x;
         dy := Dust[s1,0].y - Dust[s3,0].y;
         dz := Dust[s1,0].z - Dust[s3,0].z;
         reduce := OPTIM / m3;

         Dust[s3,0].x := (dx * reduce) +Dust[s3,0].x;
         Dust[s3,0].y := (dy * reduce) +Dust[s3,0].y;
         Dust[s3,0].z := (dz * reduce) +Dust[s3,0].z;

         Dust[s3,0] := Normalize(Dust[s3,0]);
      end;

      if (m4 > OPTIM) and
         (s4 <> 0) then
      begin
         inc(Small);
         dx := Dust[s1,0].x - Dust[s4,0].x;
         dy := Dust[s1,0].y - Dust[s4,0].y;
         dz := Dust[s1,0].z - Dust[s4,0].z;
         reduce := OPTIM / m4;

         Dust[s4,0].x := (dx * reduce) +Dust[s4,0].x;
         Dust[s4,0].y := (dy * reduce) +Dust[s4,0].y;
         Dust[s4,0].z := (dz * reduce) +Dust[s4,0].z;

         Dust[s4,0] := Normalize(Dust[s4,0]);
      end;
   end;

end;

procedure TForm1.VectorDust;
const
   deFRACT = 100;
var
   count, count2: integer;
   x1,y1,z1, x2,y2,z2,
   xd,yd,zd, d: GLFloat;
begin
   for count := 1 to MAX_DUST do
   begin
      for count2 := 1 to MAX_DUST do
      begin
            x1 := Dust[count,0].x *deFRACT;
            y1 := Dust[count,0].y *deFRACT;
            z1 := Dust[count,0].z *deFRACT;

            x2 := Dust[count2,0].x *deFRACT;
            y2 := Dust[count2,0].y *deFRACT;
            z2 := Dust[count2,0].z *deFRACT;

         xd := x2-x1;
         yd := y2-y1;
         zd := z2-z1;
         d := xd*xd + yd*yd + zd*zd;
//         if d < 100 then d := 1;
         if d <> 0 then
            d := 1/d;
         with Dust[count2,1] do
         begin
            x := x -(xd * d);
            Dust[count2,0].x := Dust[count2,0].x + x;
            y := y -(yd * d);
            Dust[count2,0].y := Dust[count2,0].y + y;
            z := z -(zd * d);
            Dust[count2,0].z := Dust[count2,0].z + z;
         end;
      end;
         if count = 100 then
            form1.Caption := FloatToStr(dust[count-1,1].x)
             +'   '+FloatToStr(dust[count-1,1].y)
             +'   '+FloatToStr(dust[count-1,1].z)
             ;
   end;
end;

procedure TForm1.BuildTerrain;
var
   row, ix: integer;
   q: GLfloat;
begin
   Randomize;
   for row := -32 to 32 do
   begin
      q := -17.0;
      for ix := 1 to 66 do
      with ter[row,ix] do
      begin
         y := -1.5+0.001 * Random(500);
         if odd(ix) then
         begin
            q := q+1.0;
            x := q+0.1-0.001 * Random(201);
            z := 0.001 * Random(450)+row;
         end else begin
            x := q;
            z := 1.0 - 0.001 *Random(450)+row;
         end;
      end;
   end;

   for row := -32 to 31 do
      for ix := 1 to 33 do
         ter[row+1,2*ix-1] := ter[row,2*ix];
end;

procedure TForm1.DrawTerrain;
var
   ix, row, extra, start, finish: integer;
   s: GLfloat;
   t: array[false..true] of GLfloat;
   tt: boolean;
begin
   extra := round(abs(10.0*cos(theta)));
   start := round(-wz);
   finish := start;
   start := start-10-extra;
   if start <-32 then start := -32;
   finish := finish+10+extra;
   if finish > 32 then finish := 32;

   glTexImage2D(GL_TEXTURE_2D,Level,ColorComps,sGrass,tGrass,Border,
            GL_RGB,GL_UNSIGNED_BYTE,Grass);
   glEnable(GL_TEXTURE_2D);
   for row := start to finish do
   begin
      s := 0.0;
      t[false] := 0.0;
      t[true] := 2.0;
      tt := false;
      glBegin(GL_TRIANGLE_STRIP);
      for ix := 1 to 66 do
      with ter[row,ix] do
      begin
         glNormal3f(0.0,1.0,0.0);
         glTexCoord2f(s,t[tt]);
         glVertex3f(x,y,z);
         if tt then
            s := s+1.0;
         tt := not tt;
      end;
      glEnd;
   end;
   glDisable(GL_TEXTURE_2D);
end;

procedure TForm1.BuildWall;
var
   q: GLfloat;
   ix: integer;
begin
   q := -2.0;
   for ix := 1 to 14 do
   with wall[ix] do
   if odd(ix) then
   begin
      q := q+1.0;
      x := q;
      y := +1.0;
   end else begin
      x := q;
      y := -1.0;
   end;

   q := q+1.0;
   for ix := 15 to 28 do
   with wall[ix] do
   if odd(ix) then
   begin
      q := q-1.0;
      x := q;
      y := 1.0;
      z := -0.5;
   end else begin
      x := q;
      y := -1.0;
      z := -0.5;
   end;
end;

procedure TForm1.Cube;
begin
   glNormal3f(0.0,0.0,1.0);
   glBegin(GL_POLYGON);  {near face}
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(+0.4,-0.4,+0.4);
      glVertex3f(+0.4,+0.4,+0.4);
      glVertex3f(-0.4,+0.4,+0.4);
   glEnd;

   glNormal3f(1.0,0.0,0.0);
   glBegin(GL_POLYGON);  {right face}
      glVertex3f(+0.4,-0.4,+0.4);
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(+0.4,+0.4,-0.4);
      glVertex3f(+0.4,+0.4,+0.4);
   glEnd;

   glNormal3f(-1.0,0.0,0.0);
   glBegin(GL_POLYGON);  {left face}
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(-0.4,+0.4,+0.4);
      glVertex3f(-0.4,+0.4,-0.4);
   glEnd;

   glNormal3f(0.0,0.0,-1.0);
   glBegin(GL_POLYGON);  {back face}
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(-0.4,+0.4,-0.4);
      glVertex3f(+0.4,+0.4,-0.4);
   glEnd;

   glNormal3f(0.0,-1.0,0.0);
   glBegin(GL_POLYGON);  {bottom face}
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(+0.4,-0.4,+0.4);
   glEnd;

   glNormal3f(0.0,1.0,0.0);
   glBegin(GL_POLYGON);  {top face}
      glVertex3f(-0.4,+0.4,+0.4);
      glVertex3f(+0.4,+0.4,+0.4);
      glVertex3f(+0.4,+0.4,-0.4);
      glVertex3f(-0.4,+0.4,-0.4);
   glEnd;
end;

procedure TForm1.CylinderBase;
var
   around: integer;
   pi18: double;
begin
   pi18 := pi/18.0;
   for around := 0 to 35 do
   begin
      circleX[around] := 0.4 * sin(around*pi18);
      circleY[around] := 0.4 * cos(around*pi18);
   end;
end;

procedure TForm1.Cylinder;
var
   around: integer;
begin
   glBegin(GL_TRIANGLE_STRIP);
     for around := 0 to 35 do
     begin
        glNormal3f(circleX[around],circleY[around],0.0);
        glVertex3f(circleX[around],circleY[around],0.0);
        glVertex3f(circleX[around],circleY[around],1.0);
     end;
     glNormal3f(circleX[0],circleY[0],0.0);
     glVertex3f(circleX[0],circleY[0],0.0);
     glVertex3f(circleX[0],circleY[0],1.0);
   glEnd;
end;

procedure TForm1.Idle(Sender: TObject; var Done: boolean);
begin
//
end;


procedure TForm1.Adjust;
var
{   viewerx,
   viewerz,
   minX,
   deltax,
   deltaz,
   distance, }
   savex,
   savey,
   savez: GLfloat;
{   ix,
   start,
   finish,
   strip : integer;}

function restore: boolean;
   begin
      result := true;
      wx := savex;
      wy := saveY;
      wz := savez;
   end;
begin
   if MY_UP then tran := +speed;
   if MY_DOWN then  tran := -speed;
   if MY_LEFT then  rot := -rspeed;
   if MY_RIGHT then  rot := +rspeed;
   if MY_W then alt := alt -speed;
   if MY_S then alt := alt +speed;

   angl := angl+rot;
   if angl > 360.0 then angl := angl -360.0;
   if angl <   0.0 then angl := angl +360.0;

   theta := angl*pi180;
   savex := wx;
   wx := wx-tran*sin(theta);
   savez := wz;
   wz := wz+tran*cos(theta);

{   strip := round(-wz);
   start := round(-wx)+34;
   finish := start+6;
   dec(start,5);
   minx := 99999.9;
   savey := wy;

   if (finish<1) or (start> 66) or (strip<-32) or (strip>32) then
      wy := 0
   else
   for ix := start to finish do
   with ter[strip,ix] do
   begin
      deltaX := abs(x-wx);
      if deltaX<minx then
      begin
         minx := deltaX;
         wy := -1.45-y;
      end;
   end;                }
   wy := wy +alt;

{   viewerx := -wx;
   viewerz := -wz;

   with TransformPole do
   begin
      deltax := tranX-viewerx;
      deltaz := tranZ-viewerz;
      distance := sqrt(deltax*deltax+deltaz*deltaz);
      if (distance< 2.115) and restore then
         exit;
   end;

   with TransformWall do
   begin
      ix := 0;
      repeat
         inc(ix,2);
         with wall[ix] do
         begin
            deltax := x-viewerx;
            deltaz := Tranz+z-viewerz;
         end;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;
      until ix=28;
   end;

   with TransformGlobe do
   begin
      deltax := TranX -viewerx;
      deltaz := Tranz -viewerz;
      distance := sqrt(deltax*deltax+deltaz*deltaz);
      if (distance <4.14) and restore then
         exit;
   end;
}
{   with TransformWall do
   begin
      deltax := TranX-viewerx;
      deltaz := TranZ-viewerz;
      distance := sqrt(deltax*deltax+deltaz*deltaz);
      if (distance < 2.67) then
      begin
         if (distance<2.42) and restore then
            exit;
         deltax := TranX+0.4-viewerx;
         deltaz := tranz+0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;

         deltax := TranX+0.4-viewerx;
         deltaz := tranz-0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;

         deltax := TranX-0.4-viewerx;
         deltaz := tranz+0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;

         deltax := TranX-0.4-viewerx;
         deltaz := tranz-0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;
      end;
   end;}

end;

procedure TForm1.TRansformPolePaint(Sender: TObject);
begin
   MaterialPole.Apply;
   Cylinder;
end;

procedure TForm1.Opening;
begin
   glEnable(GL_TEXTURE_2D);

   glTexImage2D( GL_TEXTURE_2D, Level, ColorComps,sBulk,tBulk,Border,
                GL_RGB,GL_UNSIGNED_BYTE,bulk);

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.0, 7.0);  glVertex3f(-1.0,-1.0,-2.0);
      glTexCoord2f( 1.0, 7.0);  glVertex3f(-0.7,-1.0,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f(-0.7, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f(-0.7, 1.0,-2.0);
      glTexCoord2f( 0.0, 1.0);  glVertex3f(-0.7, 0.7,-2.0);
      glTexCoord2f( 1.0, 1.0);  glVertex3f(-0.4, 0.7,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f(-0.4, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f( 0.7, 1.0,-2.0);
      glTexCoord2f( 0.0, 7.0);  glVertex3f( 0.7,-1.0,-2.0);
      glTexCoord2f( 1.0, 7.0);  glVertex3f( 1.0,-1.0,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f( 1.0, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f( 0.4, 1.0,-2.0);
      glTexCoord2f( 0.0, 1.0);  glVertex3f( 0.4, 0.7,-2.0);
      glTexCoord2f( 1.0, 1.0);  glVertex3f( 0.7, 0.7,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f( 0.7, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.4, 1.0);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.4, 0.7);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.7, 0.7);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.7, 1.0);  glVertex3f(-1.0, 1.0,-2.0);
   glEnd;

   glDisable(GL_TEXTURE_2D);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
    FreeMem(bulk);
    FreeMem(Brick);
    FreeMem(world);
    freeMem(grass);
    gluDeleteQuadric(globe);
end;

procedure TForm1.TRansformGlobePaint(Sender: TObject);
begin
   glPushMatrix;
      glRotatef(ang,0.0,0.0,1.0);

      glEnable(GL_TEXTURE_2D);
      glTexImage2D(GL_TEXTURE_2D,Level,ColorComps,sworld,tWorld,Border,
                   GL_RGB,GL_UNSIGNED_BYTE, world);
      MaterialGlobe.Apply;
      gluSphere(globe, 2.0,18,12);
      glDisable(GL_TEXTURE_2D);

   glPopMatrix;
end;

procedure TForm1.TRansformLandPaint(Sender: TObject);
begin
   MaterialLand.Apply;
   DrawTerrain;
end;

procedure TForm1.TransformCanvasPaint(Sender: TObject);
begin
   MaterialCanvas.Apply;
   Cube;
end;

procedure TForm1.TransformDustPaint(Sender: TObject);
begin
   MaterialDust.Apply;
   DrawDust;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
const
   rollover = 10;
   mspersec = 1000.0;
   half = 1/30000.0;
   fullrot = 360.0;
var
   deltaAngle,
   averageTime,
   timedif: GLfloat;
begin
   Adjust;
   inc(frames);
   elapsed := TimeGetTime;
   timedif := (elapsed - start);
   averageTime := timedif/frames;
   deltaAngle := fullrot*averageTime*half;
   ang := ang+deltaAngle;
   if ang>= 360.0 then
      ang := ang-360.0;

   if frames >= rollover then
   begin
//      Form1.Caption := Format('fps =%0.1f   Speed =%0.2f   Rot =%0.2f',
//               [rollover*mspersec/(elapsed-start)
//                , speed, rot]);
      start := elapsed;
      frames := 0;
   end;

//   VectorDust;
   SpreadDust;
   if last_small <> small then
      CollectDust;
   FormPaint(Self);
end;

end.
