unit Real0P;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Light, Material, Transfrm, cOpenGL, OpenGL, MMsystem;

type
  TForm1 = class(TForm)
    OpenGL1: TOpenGL;
    TRansformLight: TTRansform;
    TRansformFloor: TTRansform;
    TRansformWall: TTRansform;
    MaterialLight: TMaterial;
    MaterialFloor: TMaterial;
    MaterialWall: TMaterial;
    Light1: TLight;
    TRansformPole: TTRansform;
    MaterialPole: TMaterial;
    MaterialOpen: TMaterial;
    TRansformGlobe: TTRansform;
    MaterialGlobe: TMaterial;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure TRansformLightPaint(Sender: TObject);
    procedure TRansformFloorPaint(Sender: TObject);
    procedure TRansformWallPaint(Sender: TObject);
    procedure TRansformPolePaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TRansformGlobePaint(Sender: TObject);
  private
     ang, angl, rspeed, speed, tran, rot, wx, wz: GLfloat;
     frames, elapsed, start: integer;
     MY_UP, MY_DOWN, MY_LEFT, MY_RIGHT: Boolean;
     procedure BuildWall;
     procedure Cube;
     procedure CylinderBase;
     procedure Cylinder;
     procedure Opening;
     procedure Idle(Sender: TObject; var Done: boolean);
     procedure Adjust;
    { Private declarations }
  public
    { Public declarations }
  end;

type
   TPoint = packed record
     x,y,z: GLfloat;
   end;

var
  wall: array[1..28] of TPoint;
  Form1: TForm1;

implementation

{$R *.DFM}

const
   Level = 0;
   Border = 0;
   ColorComps =3;

type
   TRGB = packed record
      r,g,b: byte;
   end;

   TWrap = array[0..0] of TRGB;

var
   pi180: double = pi/180;
   circleX, circleY : array [0..35] of glFloat;
   sBulk, tBulk,
   sBrick, tBrick,
   sWorld, tWorld: GLsizei;
   globe, world, bulk, brick: pointer;

function ReadBitmap(const FilePath: string; var sWidth,tHeight:GLsizei): pointer;
const
   szh = SizeOf(TBitmapFileHeader);
   szi = SizeOf(TBitmapInfoHeader);
var
   bmpfile: file;
   bfh: TBitmapFileHeader;
   bmi: TBitmapInfoHeader;
   t: byte;
   x1, size: integer;
begin
   assignFile(bmpfile, FilePath);
   reset(bmpfile,1);
   size := FileSize(bmpfile)-szh-szi;
   blockread(bmpfile, bfh, szh);
   if Bfh.BfType<>$4D42 then
      raise EInvalidGraphic.Create('invalid bitmap');
   blockread(bmpfile,bmi,szi);
   with bmi do
   begin
      sWidth := biWidth;
      tHeight := biHeight;
   end;
   getmem(result,size);
   blockRead(bmpfile,result^,size);
   for x1 := 0 to sWidth * tHeight -1 do
   with TWrap(result^)[x1] do
   begin
      t := r;
      r := b;
      b := t;
   end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   glPixelStorei(GL_UNPACK_ALIGNMENT,1);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
   glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);//GL_DECAL);
   bulk := readBitmap('Fig13-07.bmp',sBulk,tBulk);
   Brick := readBitmap('Fig13-10.bmp',sBrick,tBrick);
  // World := readBitmap('AreaMap.bmp',sWorld,tWorld);
   World := readBitmap('Fig13-21.bmp',sWorld,tWorld);
  // World := readBitmap('Fig13-16.bmp',sWorld,tWorld);
   Application.OnIdle := Idle;
   cylinderBase;
   buildWall;
   globe := gluNewQuadric;
   gluQuadricDrawStyle(globe,GLU_FILL);
   gluQuadricTexture(globe,byteBool(GL_TRUE));
   start := TimeGetTime;
   speed := 0.2;
   rspeed := 7.0;

   MY_UP := False;
   MY_DOWN := False;
   MY_LEFT := False;
   MY_RIGHT := False;

end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   case Key of
      VK_UP: MY_UP := TRUE;
      VK_DOWN: MY_DOWN := TRUE;
      VK_LEFT: MY_LEFT := TRUE;
      VK_RIGHT: MY_RIGHT := TRUE;
      VK_ESCAPE: Close;
      ord('R'): Begin
      if ssShift in Shift then
         rspeed := rspeed +0.01
      else begin
         if rspeed > 0.01 then
            rspeed := rspeed -0.01;
         end;
      end;
      ord('S'): begin
         if ssShift in Shift then
            speed := speed +0.001
         else begin
            if speed > 0.001 then
               speed := speed -0.001;
            end;
         end;
      end; {case}

end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   case Key of
      VK_UP: begin
             MY_UP := False;
             tran := 0.0;
             end;
      VK_DOWN: begin
             MY_DOWN := False;
             tran := 0.0;
             end;
      VK_LEFT: begin
             MY_LEFT := False;
             rot := 0.0;
             end;
      VK_RIGHT: begin
             MY_RIGHT := False;
             rot := 0.0;
             end;
   end;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
   glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;

   MaterialOpen.Apply;
   Opening;

   glRotatef(angl,0.0,1.0,0.0);
   glTranslatef(wx,0.0,wz);

   TransformLight.Apply;
   TransformWall.Apply;
   TransformPole.Apply;
   TransformGlobe.Apply;
   TransformFloor.Apply;

   OpenGL1.Swap;
   OpenGL1.ErrorCheck('formpaint');
end;

procedure TForm1.FormResize(Sender: TObject);
begin
   if OpenGL1.Smaller then
      FormPaint(Sender);

end;

procedure TForm1.TRansformLightPaint(Sender: TObject);
begin
   Light1.Apply;
   MaterialLight.Apply;
   Cube;

end;

procedure TForm1.TRansformFloorPaint(Sender: TObject);
begin
   MaterialFloor.Apply;
   glNormal3f(0.0,1.0,0.0);
   glBegin(GL_POLYGON);
      glVertex3f(-29.0,0.0,-29.0);
      glVertex3f(-29.0,0.0,+29.0);
      glVertex3f(+29.0,0.0,+29.0);
      glVertex3f(+29.0,0.0,-29.0);
   glEnd;

end;

procedure TForm1.TRansformWallPaint(Sender: TObject);
var
   ix: integer;
   s: GLfloat;
   t: array[false..true] of GLfloat;
   tt: boolean;
begin
   glEnable(GL_TEXTURE_2D);
   glTexImage2D(GL_TEXTURE_2D,Level,ColorComps,sBrick,tBrick,Border,
                GL_RGB,GL_UNSIGNED_BYTE,Brick);

   MaterialWall.Apply;

   s := 0.0;
   t[false] := 0.0;
   t[true] := 4.0;
   tt := false;

   glBegin(GL_TRIANGLE_STRIP);
     for ix := 1 to 28 do
     with wall[ix] do
     begin
        glTexCoord2f(s,t[tt]);
        glVertex3f(x,y,z);
        if tt then
           s := s+2.0;
        tt := not tt;
     end;

     for ix := 1 to 2 do
     with wall[ix] do
     begin
        glTexCoord2f(s,t[tt]);
        glVertex3f(x,y,z);
        if tt then
           s := s+2.0;
        tt := not tt;
     end;
  glEnd;
  glDisable(GL_TEXTURE_2D);

//   Cube;
end;

procedure TForm1.BuildWall;
var
   q: GLfloat;
   ix: integer;
begin
   q := -2.0;
   for ix := 1 to 14 do
   with wall[ix] do
   if odd(ix) then
   begin
      q := q+1.0;
      x := q;
      y := +1.0;
   end else begin
      x := q;
      y := -1.0;
   end;

   q := q+1.0;
   for ix := 15 to 28 do
   with wall[ix] do
   if odd(ix) then
   begin
      q := q-1.0;
      x := q;
      y := 1.0;
      z := -0.5;
   end else begin
      x := q;
      y := -1.0;
      z := -0.5;
   end;
end;

procedure TForm1.Cube;
begin
   glNormal3f(0.0,0.0,1.0);
   glBegin(GL_POLYGON);  {near face}
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(+0.4,-0.4,+0.4);
      glVertex3f(+0.4,+0.4,+0.4);
      glVertex3f(-0.4,+0.4,+0.4);
   glEnd;

   glNormal3f(1.0,0.0,0.0);
   glBegin(GL_POLYGON);  {right face}
      glVertex3f(+0.4,-0.4,+0.4);
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(+0.4,+0.4,-0.4);
      glVertex3f(+0.4,+0.4,+0.4);
   glEnd;

   glNormal3f(-1.0,0.0,0.0);
   glBegin(GL_POLYGON);  {left face}
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(-0.4,+0.4,+0.4);
      glVertex3f(-0.4,+0.4,-0.4);
   glEnd;

   glNormal3f(0.0,0.0,-1.0);
   glBegin(GL_POLYGON);  {back face}
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(-0.4,+0.4,-0.4);
      glVertex3f(+0.4,+0.4,-0.4);
   glEnd;

   glNormal3f(0.0,-1.0,0.0);
   glBegin(GL_POLYGON);  {bottom face}
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(+0.4,-0.4,+0.4);
   glEnd;

   glNormal3f(0.0,1.0,0.0);
   glBegin(GL_POLYGON);  {top face}
      glVertex3f(-0.4,+0.4,+0.4);
      glVertex3f(+0.4,+0.4,+0.4);
      glVertex3f(+0.4,+0.4,-0.4);
      glVertex3f(-0.4,+0.4,-0.4);
   glEnd;
end;

procedure TForm1.CylinderBase;
var
   around: integer;
   pi18: double;
begin
   pi18 := pi/18.0;
   for around := 0 to 35 do
   begin
      circleX[around] := 0.4 * sin(around*pi18);
      circleY[around] := 0.4 * cos(around*pi18);
   end;
end;

procedure TForm1.Cylinder;
var
   around: integer;
begin
   glBegin(GL_TRIANGLE_STRIP);
     for around := 0 to 35 do
     begin
        glNormal3f(circleX[around],circleY[around],0.0);
        glVertex3f(circleX[around],circleY[around],0.0);
        glVertex3f(circleX[around],circleY[around],1.0);
     end;
     glNormal3f(circleX[0],circleY[0],0.0);
     glVertex3f(circleX[0],circleY[0],0.0);
     glVertex3f(circleX[0],circleY[0],1.0);
   glEnd;
end;

procedure TForm1.Idle(Sender: TObject; var Done: boolean);
const
   rollover = 10;
   mspersec = 1000.0;
   half = 1/30000.0;
   fullrot = 360.0;
var
   deltaAngle,
   averageTime,
   timedif: GLfloat;
begin
   Done := false;
   Adjust;
   inc(frames);
   elapsed := TimeGetTime;
   timedif := (elapsed - start);
   averageTime := timedif/frames;
   deltaAngle := fullrot*averageTime*half;
   ang := ang+deltaAngle;
   if ang>= 360.0 then
      ang := ang-360.0;

   if frames >= rollover then
   begin
      Form1.Caption := Format('fps =%0.1f   Speed =%0.2f   Rot =%0.2f',
               [rollover*mspersec/(elapsed-start)
                , speed, rot]);
      start := elapsed;
      frames := 0;
   end else
      formPaint(nil);
end;


procedure TForm1.Adjust;
var
   theta,
   viewerx,
   viewerz,
   deltax,
   deltaz,
   distance,
   savex,
   savez: GLfloat;
   ix : integer;

function restore: boolean;
   begin
      result := true;
      wx := savex;
      wz := savez;
   end;
begin
   if MY_UP then tran := +speed;
   if MY_DOWN then  tran := -speed;
   if MY_LEFT then  rot := -rspeed;
   if MY_RIGHT then  rot := +rspeed;

   angl := angl+rot;
   theta := angl*pi180;
   savex := wx;
   wx := wx-tran*sin(theta);
   savez := wz;
   wz := wz+tran*cos(theta);

   viewerx := -wx;
   viewerz := -wz;

   with TransformPole do
   begin
      deltax := tranX-viewerx;
      deltaz := tranZ-viewerz;
      distance := sqrt(deltax*deltax+deltaz*deltaz);
      if (distance< 2.115) and restore then
         exit;
   end;

   with TransformWall do
   begin
      ix := 0;
      repeat
         inc(ix,2);
         with wall[ix] do
         begin
            deltax := x-viewerx;
            deltaz := Tranz+z-viewerz;
         end;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;
      until ix=28;
   end;

   with TransformGlobe do
   begin
      deltax := TranX -viewerx;
      deltaz := Tranz -viewerz;
      distance := sqrt(deltax*deltax+deltaz*deltaz);
      if (distance <4.14) and restore then
         exit;
   end;

{   with TransformWall do
   begin
      deltax := TranX-viewerx;
      deltaz := TranZ-viewerz;
      distance := sqrt(deltax*deltax+deltaz*deltaz);
      if (distance < 2.67) then
      begin
         if (distance<2.42) and restore then
            exit;
         deltax := TranX+0.4-viewerx;
         deltaz := tranz+0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;

         deltax := TranX+0.4-viewerx;
         deltaz := tranz-0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;

         deltax := TranX-0.4-viewerx;
         deltaz := tranz+0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;

         deltax := TranX-0.4-viewerx;
         deltaz := tranz-0.4-viewerz;
         distance := sqrt(deltax*deltax+deltaz*deltaz);
         if (distance<2.42) and restore then
            exit;
      end;
   end;}

end;

procedure TForm1.TRansformPolePaint(Sender: TObject);
begin
   MaterialPole.Apply;
   Cylinder;
end;

procedure TForm1.Opening;
begin
   glEnable(GL_TEXTURE_2D);

   glTexImage2D( GL_TEXTURE_2D, Level, ColorComps,sBulk,tBulk,Border,
                GL_RGB,GL_UNSIGNED_BYTE,bulk);

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.0, 7.0);  glVertex3f(-1.0,-1.0,-2.0);
      glTexCoord2f( 1.0, 7.0);  glVertex3f(-0.7,-1.0,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f(-0.7, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f(-0.7, 1.0,-2.0);
      glTexCoord2f( 0.0, 1.0);  glVertex3f(-0.7, 0.7,-2.0);
      glTexCoord2f( 1.0, 1.0);  glVertex3f(-0.4, 0.7,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f(-0.4, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f( 0.7, 1.0,-2.0);
      glTexCoord2f( 0.0, 7.0);  glVertex3f( 0.7,-1.0,-2.0);
      glTexCoord2f( 1.0, 7.0);  glVertex3f( 1.0,-1.0,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f( 1.0, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.0, 0.0);  glVertex3f( 0.4, 1.0,-2.0);
      glTexCoord2f( 0.0, 1.0);  glVertex3f( 0.4, 0.7,-2.0);
      glTexCoord2f( 1.0, 1.0);  glVertex3f( 0.7, 0.7,-2.0);
      glTexCoord2f( 1.0, 0.0);  glVertex3f( 0.7, 1.0,-2.0);
   glEnd;

   glBegin(GL_POLYGON);
      glTexCoord2f( 0.4, 1.0);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.4, 0.7);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.7, 0.7);  glVertex3f(-1.0, 1.0,-2.0);
      glTexCoord2f( 0.7, 1.0);  glVertex3f(-1.0, 1.0,-2.0);
   glEnd;

   glDisable(GL_TEXTURE_2D);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
    FreeMem(bulk);
    FreeMem(Brick);
    FreeMem(world);
    gluDeleteQuadric(globe);
end;

procedure TForm1.TRansformGlobePaint(Sender: TObject);
begin
   glPushMatrix;
      glRotatef(ang,0.0,0.0,1.0);

      glEnable(GL_TEXTURE_2D);
      glTexImage2D(GL_TEXTURE_2D,Level,ColorComps,sworld,tWorld,Border,
                   GL_RGB,GL_UNSIGNED_BYTE, world);
      MaterialGlobe.Apply;
      gluSphere(globe, 2.0,18,12);
      glDisable(GL_TEXTURE_2D);

   glPopMatrix;
end;

end.
