unit Simple0P;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Light, Material, Transfrm, cOpenGL, OpenGL, MMsystem;

type
  TForm1 = class(TForm)
    OpenGL1: TOpenGL;
    TRansformLight: TTRansform;
    TRansformFloor: TTRansform;
    TRansformCube: TTRansform;
    MaterialLight: TMaterial;
    MaterialFloor: TMaterial;
    MaterialCube: TMaterial;
    Light1: TLight;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure TRansformLightPaint(Sender: TObject);
    procedure TRansformFloorPaint(Sender: TObject);
    procedure TRansformCubePaint(Sender: TObject);
  private
     angl, rspeed, speed, tran, rot, wx, wz: GLfloat;
     frames, elapsed, start: integer;
     procedure Cube;
     procedure Idle(Sender: TObject; var Done: boolean);
     procedure Adjust;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  pi180: double = pi/180;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
   Application.OnIdle := Idle;
   start := TimeGetTime;

end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   case Key of
      VK_UP: tran := +speed;
      VK_DOWN: tran := -speed;
      VK_LEFT: rot := -rspeed;
      VK_RIGHT: rot := +rspeed;
      VK_ESCAPE: Close;
      ord('R'): Begin
      if ssShift in Shift then
         rspeed := rspeed +0.01
      else begin
         if rspeed > 0.01 then
            rspeed := rspeed -0.01;
         end;
      end;
      ord('S'): begin
         if ssShift in Shift then
            speed := speed +0.001
         else begin
            if speed > 0.001 then
               speed := speed -0.001;
            end;
         end;
      end; {case}

end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   tran := 0.0;
   rot := 0.0;

end;

procedure TForm1.FormPaint(Sender: TObject);
begin
   glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;
   glRotatef(angl,0.0,1.0,0.0);
   glTranslatef(wx,0.0,wz);

   TransformLight.Apply;
   TransformFloor.Apply;
   TransformCube.Apply;

   OpenGL1.Swap;
   OpenGL1.ErrorCheck('formpaint');

end;

procedure TForm1.FormResize(Sender: TObject);
begin
   if OpenGL1.Smaller then
      FormPaint(Sender);

end;

procedure TForm1.TRansformLightPaint(Sender: TObject);
begin
   Light1.Apply;
   MaterialLight.Apply;
   Cube;

end;

procedure TForm1.TRansformFloorPaint(Sender: TObject);
begin
   MaterialFloor.Apply;
   glNormal3f(0.0,1.0,0.0);
   glBegin(GL_POLYGON);
      glVertex3f(-29.0,0.0,-29.0);
      glVertex3f(-29.0,0.0,+29.0);
      glVertex3f(+29.0,0.0,+29.0);
      glVertex3f(+29.0,0.0,-29.0);
   glEnd;

end;

procedure TForm1.TRansformCubePaint(Sender: TObject);
begin
   MaterialCube.Apply;
   Cube;
end;

procedure TForm1.Cube;
begin
   glNormal3f(0.0,0.0,1.0);
   glBegin(GL_POLYGON);  {near face}
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(+0.4,-0.4,+0.4);
      glVertex3f(+0.4,+0.4,+0.4);
      glVertex3f(-0.4,+0.4,+0.4);
   glEnd;

   glNormal3f(1.0,0.0,0.0);
   glBegin(GL_POLYGON);  {right face}
      glVertex3f(+0.4,-0.4,+0.4);
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(+0.4,+0.4,-0.4);
      glVertex3f(+0.4,+0.4,+0.4);
   glEnd;

   glNormal3f(-1.0,0.0,0.0);
   glBegin(GL_POLYGON);  {left face}
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(-0.4,+0.4,+0.4);
      glVertex3f(-0.4,+0.4,-0.4);
   glEnd;

   glNormal3f(0.0,0.0,-1.0);
   glBegin(GL_POLYGON);  {back face}
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(-0.4,+0.4,-0.4);
      glVertex3f(+0.4,+0.4,-0.4);
   glEnd;

   glNormal3f(0.0,-1.0,0.0);
   glBegin(GL_POLYGON);  {bottom face}
      glVertex3f(-0.4,-0.4,+0.4);
      glVertex3f(-0.4,-0.4,-0.4);
      glVertex3f(+0.4,-0.4,-0.4);
      glVertex3f(+0.4,-0.4,+0.4);
   glEnd;

   glNormal3f(0.0,1.0,0.0);
   glBegin(GL_POLYGON);  {top face}
      glVertex3f(-0.4,+0.4,+0.4);
      glVertex3f(+0.4,+0.4,+0.4);
      glVertex3f(+0.4,+0.4,-0.4);
      glVertex3f(-0.4,+0.4,-0.4);
   glEnd;
end;

procedure TForm1.Idle(Sender: TObject; var Done: boolean);
const
   rollover = 10;
   mspersec = 1000.0;
begin
   Done := false;
   Adjust;
   inc(frames);
   if frames >= rollover then
   begin
      elapsed := TimeGetTime;
      Form1.Caption := Format('fps =%0.1f   Speed =%0.2f   Rot =%0.2f',
               [rollover*mspersec/(elapsed-start)
                , speed, rot]);
      start := elapsed;
      frames := 0;
   end else
      formPaint(nil);
end;

procedure TForm1.Adjust;
var
   theta: GLfloat;
begin
   angl := angl+rot;
   theta := angl*pi180;
   wx := wx-tran*sin(theta);
   wz := wz+tran*cos(theta);
end;

end.
