unit spot1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OpenGL, Menus;

type
  TLightDesc = packed record
    red,
    green,
    blue,
    alpha: glFloat;
  end;
  TLightPlace = packed record
    x,y,z,w: glFloat;
  end;
type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    GLContext: HGLRC;
    glDC: HDC;
    errorCode: GLenum;
    openGLReady: boolean;
    oldw,
    oldh: integer;
    x1,y1,z1,x2,y2,z2,sign,angle,xr,yr,zr: glFloat;
    lastMouseX,lastMouseY: integer;
    SpotColor,HexColor,
    DiffDef,AmbientDef: TLightDesc;//array[0..3] of glFloat;
    LightPlace: TLightPlace;
    procedure ExceptionGL(Sender:TObject;E:Exception);
    procedure Cube(matColor:TLightDesc);
    procedure Hex(matColor:TLightDesc);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
  pfd: TPixelFormatDescriptor;
  FormatIndex: integer;
begin
  Application.OnException := ExceptionGL;
  fillchar(pfd,SizeOf(pfd),0);
  with pfd do
  begin
    nSize      := SizeOf(pfd);
    nVersion   := 1; {The current version of the desccriptor is 1}
    dwFlags    := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL;
    iPixelType := PFD_TYPE_RGBA;
    cColorBits := 24; {support 24-bit color}
    cDepthBits := 32; {depth of z-axis}
    iLayerType := PFD_MAIN_PLANE;
  end; {with}
  glDC := getDC(handle);
  FormatIndex := ChoosePixelFormat(glDC,@pfd);
  if FormatIndex=0 then
    raise Exception.Create('ChoosePixelFormat failed '+
      IntToStr(GetLastError));

  if not SetPixelFormat(glDC,FormatIndex,@pfd) then
    raise Exception.Create('SetPixelFormat failed '+
      IntToStr(GetLastError));

  GLContext := wglCreateContext(glDC);
  if GLContext=0 then
    raise Exception.Create('wglCreateContext failed '+
      IntToStr(GetLastError));

  if not wglMakeCurrent(glDC,GLContext) then
    raise Exception.Create('wglMakeCurrent failed '+
      IntToStr(GetLastError));

  OpenGLReady := true;
  oldw := ClientWidth;
  oldh := ClientHeight;
  glMatrixMode(GL_PROJECTION);
  glFrustum(-10.0,10.0,-10.0,10.0,2.0,10837.0);
  glEnable(GL_DEPTH_TEST);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
  glTranslatef(0.0,0.0,-2.4);

  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);
  glEnable(GL_LIGHTING);
  with AmbientDef do
  begin
    red := 0.2;
    green := 0.2;
    blue := 0.6;
    alpha := 1.0;
  end;
  with DiffDef do
  begin
    red := 1.0;
    green := 1.0;
    blue := 1.0;
    alpha := 1.0;
  end;
  with HexColor do
  begin
    red := 0.5;
    green := 0.0;
    blue := 0.8;
    alpha := 1.0;
  end;
  with SpotColor do
  begin
    red := 0.0;
    green := 0.5;
    blue := 0.5;
    alpha := 1.0;
  end;

  LightPlace.x := 1.0;
  LightPlace.y := 1.0;
  LightPlace.w := 1.0;
  LightPlace.z := 1.0;
  glLightfv(GL_LIGHT0,GL_AMBIENT,@AmbientDef);
  glLightfv(GL_LIGHT0,GL_DIFFUSE,@DiffDef);
  glEnable(GL_LIGHT0);

  sign := 12.0;
  angle := 0.0;
  x1 := 0.0;
  y1 := 0.0;
  z1 := -2.4;
  x2 := 0.0;
  y2 := 0.0;
  z2 := -2.4;
  xr := 0.0;
  yr := 0.0;
  zr := 90.0;
end; {FormCreate}

procedure TForm1.FormDestroy(Sender: TObject);
begin
  OpenGLReady := false;
  wglMakeCurrent(glDC,0);
  wglDeleteContext(GLContext);
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  if not openGLReady then
    exit;

  {green background}
  glClearColor(0.6,0.8,0.8,0.0);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1-0.0,y1+0.0,z1-422.8);
  Hex(HexColor);
  glPopMatrix;

 { glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(y1+112.0,x1+192.0,z1-422.8);
  Hex;
  glPopMatrix;
}
  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1+224.0,y1+384.0,z1-422.8);
  Hex(HexColor);
  glPopMatrix;

  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1-224.0,y1+384.0,z1-422.8);
  Hex(HexColor);
  glPopMatrix;

  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1-448.0,y1+0.0,z1-422.8);
  Hex(HexColor);
  glPopMatrix;

  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1-224.0,y1-384.0,z1-422.8);
  Hex(HexColor);
  glPopMatrix;

  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1+224.0,y1-384.0,z1-422.8);
  Hex(HexColor);
  glPopMatrix;

  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1+448.0,y1+0.0,z1-422.8);
  Hex(HexColor);
  glPopMatrix;

  glPushMatrix;
  glScalef(0.33,0.33,0.33);
  glTranslatef(x1+200.0,y1+100.0,z1-4.0);
  Hex(HexColor);
  glPopMatrix;

{  glPushMatrix;
  glRotatef(angle,xr,yr,zr);
  glTranslatef(x1+1.9,y1-1.6,z1-5.4);
  Cube;
  glPopMatrix; }

  glPushMatrix;
  glTranslatef(lightPlace.x+200,lightPlace.y+200,lightPlace.z);
  glLightfv(GL_LIGHT0,GL_POSITION,@LightPlace);
  glPopMatrix;

  glFlush;

  errorCode := glGetError;
  if errorCode<>GL_NO_ERROR then
    raise Exception.Create('Error in Paint'#13+
      gluErrorString(errorCode));
end; {FormPaint}

procedure TForm1.Hex(matColor:TLightDesc);
var
  diffuse,
  ambient: TLightDesc;
begin
  glMaterialfv(GL_FRONT,GL_AMBIENT,@matColor);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,@matColor);

  glBegin(GL_POLYGON);
    glVertex3f(-112.0,-64.0,+128.0);
    glVertex3f(-0.0,-128.0,+128.0);
    glVertex3f(+112.0,-64.0,+128.0);
    glVertex3f(+112.0,+64.0,+128.0);
    glVertex3f(-0.0,+128.0,+128.0);
    glVertex3f(-112.0,+64.0,+128.0);
  glEnd;

  glBegin(GL_POLYGON);
    glVertex3f(-112.0,-64.0,-128.0);
    glVertex3f(-112.0,+64.0,-128.0);
    glVertex3f(-0.0,+128.0,-128.0);
    glVertex3f(+112.0,+64.0,-128.0);
    glVertex3f(+112.0,-64.0,-128.0);
    glVertex3f(-0.0,-128.0,-128.0);
  glEnd;

  glBegin(GL_POLYGON);
    glVertex3f(-112.0,-64.0,-128.0);
    glVertex3f(-0.0,-128.0,-128.0);
    glVertex3f(-0.0,-128.0,+128.0);
    glVertex3f(-112.0,-64.0,+128.0);
  glEnd;

  glBegin(GL_POLYGON);
    glVertex3f(-0.0,-128.0,-128.0);
    glVertex3f(+112.0,-64.0,-128.0);
    glVertex3f(+112.0,-64.0,+128.0);
    glVertex3f(-0.0,-128.0,+128.0);
  glEnd;

  glBegin(GL_POLYGON);
    glVertex3f(+112.0,-64.0,-128.0);
    glVertex3f(+112.0,+64.0,-128.0);
    glVertex3f(+112.0,+64.0,+128.0);
    glVertex3f(+112.0,-64.0,+128.0);
  glEnd;

  glBegin(GL_POLYGON);
    glVertex3f(+0.0,+128.0,-128.0);
    glVertex3f(+0.0,+128.0,+128.0);
    glVertex3f(+112.0,+64.0,+128.0);
    glVertex3f(+112.0,+64.0,-128.0);
  glEnd;

  glBegin(GL_POLYGON);
    glVertex3f(-112.0,+64.0,-128.0);
    glVertex3f(-112.0,+64.0,+128.0);
    glVertex3f(+0.0,+128.0,+128.0);
    glVertex3f(+0.0,+128.0,-128.0);
  glEnd;

  glBegin(GL_POLYGON);
    glVertex3f(-112.0,-64.0,-128.0);
    glVertex3f(-112.0,-64.0,+128.0);
    glVertex3f(-112.0,+64.0,+128.0);
    glVertex3f(-112.0,+64.0,-128.0);
  glEnd;

end; {TForm1.Hex}

procedure TForm1.Cube;
begin
  glBegin(GL_POLYGON); {near face}
    glColor3f(0.5,0.0,0.0);
    glVertex3f(-0.4,-0.4,+0.4);
    glVertex3f(+0.4,-0.4,+0.4);
    glVertex3f(+0.4,+0.4,+0.4);
    glVertex3f(-0.4,+0.4,+0.4);
  glEnd;

  glBegin(GL_POLYGON); {right face}
    glColor3f(0.3,0.3,0.8);
    glVertex3f(+0.4,-0.4,+0.4);
    glVertex3f(+0.4,-0.4,-0.4);
    glVertex3f(+0.4,+0.4,-0.4);
    glVertex3f(+0.4,+0.4,+0.4);
  glEnd;

  glBegin(GL_POLYGON); {left face}
    glColor3f(0.5,0.0,0.5);
    glVertex3f(-0.4,-0.4,-0.4);
    glVertex3f(-0.4,-0.4,+0.4);
    glVertex3f(-0.4,+0.4,+0.4);
    glVertex3f(-0.4,+0.4,-0.4);
  glEnd;

  glBegin(GL_POLYGON); {back face}
    glColor3f(0.0,0.0,0.5);
    glVertex3f(+0.4,-0.4,-0.4);
    glVertex3f(-0.4,-0.4,-0.4);
    glVertex3f(-0.4,+0.4,-0.4);
    glVertex3f(+0.4,+0.4,-0.4);
  glEnd;

  glBegin(GL_POLYGON); {bottom face}
    glColor3f(0.5,0.5,0.0);
    glVertex3f(-0.4,-0.4,+0.4);
    glVertex3f(-0.4,-0.4,-0.4);
    glVertex3f(+0.4,-0.4,-0.4);
    glVertex3f(+0.4,-0.4,+0.4);
  glEnd;

  glBegin(GL_POLYGON); {top face}
    glColor3f(0.3,0.0,0.1);
    glVertex3f(-0.4,+0.4,+0.4);
    glVertex3f(+0.4,+0.4,+0.4);
    glVertex3f(+0.4,+0.4,-0.4);
    glVertex3f(-0.4,+0.4,-0.4);
  glEnd;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  if not openGLReady then
    exit;
  glViewPort(0,0,ClientWidth,ClientHeight);

  errorCode := glGetError;
  if errorCode<>GL_NO_ERROR then
    raise Exception.Create('FormResize:'+gluErrorString(errorCode));
  if (ClientWidth<=oldw) and (ClientHeight<=oldh) then
    FormPaint(Sender);
  oldh := ClientHeight;
  oldw := ClientWidth;
end;

procedure TForm1.ExceptionGL(Sender:TObject;E:Exception);
begin
  ShowMessage(E.Message);
  repeat
    errorCode := glGetError;
    if errorCode<>GL_NO_ERROR then
      showMessage(gluErrorString(errorCode));
  until errorCode=GL_NO_ERROR;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  ok: boolean;
begin
  ok := true;
  form1.Caption := inttostr(key)+'    '+FloatToStr(lightplace.z);
  case Key of
    33: angle := angle -1;
    34: angle := angle +1;
    37: xr := xr -sign;
    38: yr := yr +sign;
    39: xr := xr +sign;
    40: yr := yr -sign;
    45: z1 := z1 +20;
    46: z1 := z1 -20;
    76: lightPlace.x := lightPlace.x -sign;
    77: lightPlace.z := lightPlace.z +20;
    78: lightPlace.z := lightPlace.z -20;
    else
      ok:= false;
  end; {case}
  if ok then
    Invalidate;
end;

procedure TForm1.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  form1.Caption := 'X: '+IntToSTr(x)+'    lastX: '+inttostr(lastmousex);
  if x<lastMouseX then
    x1 := x1- 100;
  if x>lastMouseX then
    x1 := x1+ 100;
  if y<lastMouseY then
    y1 := y1+ 100;
  if y>lastMouseY then
    y1 := y1-100;
  lastMouseX := x;
  lastMouseY := y;
  invalidate;
end;

end.
